#pragma once

#include <stdarg.h>

///////////////////////////////////////////////////////////////////////////////////////////////
const int INSERT_AFTER  = 0;
const int INSERT_BEFORE = 1;

template<class T>
class CTList
{
	private:

				struct node
				{
					T	   item;
					node * next;
					node * prev;

				};

				node * m_root;
				node * m_last;

	public:
				bool (*CmpFn)(T, T);


				class iterator
				{
					node * m_iterator;

					public:

						    iterator(node * nodeptr = NULL) : m_iterator(nodeptr)
							{}

							~iterator()
							{
								m_iterator = NULL;
							}

							iterator& operator++()
							{
								m_iterator = m_iterator->next;
								return *this;
							}

							iterator& operator++(int)
							{
								/*iterator _temp = *this;
								++*this;
								return _temp;*/
								return (++*this);
							}

							iterator& operator--()
							{
								m_iterator = m_iterator->prev;
								return *this;
							}

							iterator& operator--(int)
							{
								iterator _temp = *this;
								--*this;
								return _temp;
							}

							T& operator*()
							{
								return m_iterator->item;
							}

							T& operator->()
							{
								return m_iterator->item;
							}

							bool operator!=(const iterator &r)
							{
								if (this->m_iterator == r.m_iterator)
									return false;
								else
									return true;
								
							}
							bool operator==(const iterator &r)
							{
								if (this->m_iterator == r.m_iterator)
									return true;
								else
									return false;
								
							}
							node * GetNode()
							{
								return m_iterator;
							}

						
				
				};

				CTList()
				{
					m_root = NULL;
					m_last = NULL;
				}

				~CTList();

				void operator+=(T p_item)
				{
					Insert(INSERT_AFTER, iterator(m_last), p_item);
				}

				void operator-=(iterator &p_it)
				{
					Remove(p_it);
				}

				iterator begin()
				{
					return iterator(m_root);
				}
				iterator end()
				{
					return iterator(m_last->next);
				}
				iterator last()
				{
					return iterator(m_last);
				}

				iterator InsFront(const T p_item)
				{
					return Insert(INSERT_BEFORE, iterator(m_root), p_item);
				}
				iterator InsBack(const T p_item)
				{
					return Insert(INSERT_AFTER, iterator(m_last), p_item);
				}
				void RemFront()
				{
					Remove(iterator(m_root));
				}
				void RemBack()
				{
					Remove(iterator(m_last));
				}
				void RemoveAll();

				iterator Find(T p_item);
				
				void RemoveItem(T p_item);

				bool IsEmpty()
				{
					if (m_root == NULL)
						return true;
					else
						return false;
				}

				void Sort();
				int  Count();

				iterator Insert(const int method, iterator &p_it, const T p_item);
				void Remove(iterator &p_it);
				void Swap(iterator &p_it0, iterator &p_it1);
				
};

///////////////////////////////////////////////////////////////////////////////////////////////

template<class T>
CTList<T>::~CTList()
{
	node * n = m_root;
	while (n)
	{
		node * o = n;
		n = n->next;
		delete o;

	}
	m_root = m_last = NULL;
}
///////////////////////////////////////////////////////////////////////////////////////////////
template<class T>
void CTList<T>::RemoveAll()
{
	node * n = m_root;
	while (n)
	{
		node * o = n;
		n = n->next;
		delete o;

	}

	m_root = m_last = NULL;

}
///////////////////////////////////////////////////////////////////////////////////////////////
template<class T>
CTList<T>::iterator CTList<T>::Insert(const int method, iterator &p_it, const T p_item)
{
	node * _node = p_it.GetNode();

	node * _new_node = new node;

	if (!_new_node)
	{
		return iterator(NULL);
	}

	_new_node->item = p_item;
	_new_node->next = NULL;
	_new_node->prev = NULL;

	
	if (m_root == NULL)          
	{
		m_root = _new_node;
		m_last = m_root;

		return iterator(_new_node);
	}

	if (method == INSERT_BEFORE)
	{
		if (_node == m_root)
		{
			_new_node->next = m_root;
			m_root->prev    = _new_node;
			m_root          = _new_node;

		} 
		else
		{
			_new_node->next   = _node;
			_new_node->prev   = _node->prev;
			_node->prev->next = _new_node;
			
		}
	}
	else
	{
		if (_node == m_last)
		{
			_new_node->prev = _node;
			_node->next     = _new_node;
			m_last          = _new_node;

		}
		else
		{
			_new_node->next = _node->next;
			_new_node->prev = _node;
			_node->next     = _new_node;
		}
	}

	return iterator(_new_node);

}
///////////////////////////////////////////////////////////////////////////////////////////////
template<class T>
void CTList<T>::Remove(iterator &p_it)
{
	if (p_it == NULL) return;
	
	node * _node = p_it.GetNode();
	node * _temp = _node;
	
	if (_node == m_root)
	{
		m_root       = m_root->next;
 		if (m_root != NULL) 
			m_root->prev = NULL;
		else
			m_last = NULL;

	}
	else
	if (_node == m_last)
	{
		m_last       = m_last->prev;
		if (m_last != NULL) m_last->next = NULL;
	}
	else
	{
		_node->prev->next = _node->next;
		_node->next->prev = _node->prev;
	}

	delete _temp;

}
///////////////////////////////////////////////////////////////////////////////////////////////
template<class T>
void CTList<T>::Swap(iterator &p_it0, iterator &p_it1)
{
	node * _node0 = p_it0.GetNode();
	node * _node1 = p_it1.GetNode();

	T _el_temp = _node0->item;
	_node0->item = _node1->item;
	_node1->item = _el_temp;
	
}
///////////////////////////////////////////////////////////////////////////////////////////////
template<class T>
int CTList<T>::Count()
{
	int n = 0;
	iterator it = begin();

	while (it != NULL)
	{
		++it;
		++n;
	}

	return n;
}
///////////////////////////////////////////////////////////////////////////////////////////////
template<class T>
CTList<T>::iterator CTList<T>::Find(T p_item)
{
	iterator it = begin();

	while (it != NULL && *it != p_item) ++it;

	return it;
}
///////////////////////////////////////////////////////////////////////////////////////////////
template<class T>
void CTList<T>::RemoveItem(T p_item)
{
	iterator it = Find(p_item);
	Remove(it);
}
///////////////////////////////////////////////////////////////////////////////////////////////

/* na razie tylko bbelkowe */

template<class T>
void CTList<T>::Sort()
{
	iterator it0;
	iterator it1;

	for (it0 = begin(); it0 != NULL; it0++)
	{
		for (it1 = begin(); it1 != NULL; it1++)
		{
			if (it0 != it1 && CmpFn(*it0, *it1))
				Swap(it0, it1);
		}
	}

	
}
///////////////////////////////////////////////////////////////////////////////////////////////
