#include "stdafx.h"
#include "FormCreator.h"
#include "Globals.h"

#include "GridCtrl/GridCtrl.h"
#include "Console.h"
#include "FormCreatorDoc.h"
#include "FormCreatorView.h"
#include "MultiRectTracker.h"
#include "ToolbarControls.h"

#include "DialogGrid.h"
#include "DialogTemplate.h"
#include "ColumnsDialog.h"

#include "TObject.h"
#include "PanelControl.h"

#include "TransparentWindow.h"
#include "math.h"
#include <afxrich.h>

#include <iostream>
using namespace std;

/*----------------------------------------------------------------------*\
	Funkcja    :  CFormCreatorView::DrawGrid
	Wersja     :  1.0
  ----------------------------------------------------------------------
	Opis       :  Rysuje siatk (grid) na formatce
	Wyjscie    :  void	-	
	Parametry  :
			      CRect rect	-	fragment formatki na ktrym bdzie
				                    rysowana siatka.
			      DWORD color	-	kolor siatki.
	Note       :
\*----------------------------------------------------------------------*/
void CFormCreatorView::DrawGrid(CRect rect, DWORD color)
{
		
	CDC * pDC = m_pDlg->GetDC();

	objIterator it = m_objects.begin();

	float dx = rect.right  / (float) m_nGridX;
	float dy = rect.bottom / (float) m_nGridY;

	float sx = m_pTracker->Nearest(rect.left, m_nGridX);
	float sy = m_pTracker->Nearest(rect.top,  m_nGridY);

	int start_sy = sy;

	CRect * curRect;

	while (sx <= (float) rect.right) 
	{
		sy = start_sy;

		while (sy <= (float) rect.bottom)
		{
			bool bInRect = false;

			int gx = (int) sx;
			int gy = (int) sy;
			
			it = m_objects.begin();

			/* eby unikn migotania sprawdzam czy dany punkt nie ley 
			   na jakim z obiektw */

			while (it != NULL)
			{	
				curRect = &it->m_curRect;

				if	(sy >= curRect->top  && sy <= curRect->bottom &&
					 sx >= curRect->left && sx <= curRect->right)
				{
						bInRect = true;
						break;
				}
				++it;
			}

			if (!bInRect)
				pDC->SetPixelV(gx, gy, color);

			sy += (float) m_nGridY;
		}

		sx += (float) m_nGridX;
	}
		
	m_pDlg->ReleaseDC(pDC);

}
/*----------------------------------------------------------------------*\
	Funkcja    :  CFormCreatorView::DrawNumbers
	Wersja     :  1.0
  ----------------------------------------------------------------------
	Opis       :  Rysuje numerki na obiektach
	Wyjscie    :  void
	Parametry  :
	Note       :
\*----------------------------------------------------------------------*/
void CFormCreatorView::DrawNumbers()
{
	if (!m_bShowNumbers)
		return;

	CDC * pDC = m_pDlg->GetDC();

	m_pBmpNumber = new CBitmap;
	m_pMemDC = new CDC;
	m_pBmpNumber->CreateCompatibleBitmap(pDC, 30, 25);
	m_pMemDC->CreateCompatibleDC(pDC);
	m_pMemDC->SelectObject(m_pBmpNumber);

	m_pMemDC->SelectObject(&fontNormal);

	int nNum = 1;

	_DrawNumbers(pDC, m_pMemDC, &m_objects, nNum);

	m_pMemDC->DeleteDC();
	m_pBmpNumber->DeleteObject();
	delete m_pMemDC;
	delete m_pBmpNumber;
	m_pDlg->ReleaseDC(pDC);

}
void CFormCreatorView::_DrawNumbers(CDC * pDC, CDC * pMemDC, CTList<CTObject *> * objects, int & nNum)
{

	CRect   curRect;

	CTList<CTObject *>::iterator it = objects->last();


	while (it != NULL)
	{
		
		CSize size = pMemDC->GetTextExtent("0");
		size.cx *= m_nZeros;


		curRect = GetObjectRealRect(*it, it->m_curRect);
		
		int nWidth = size.cx + 5;
		int nHeight = size.cy + 2;

		curRect.SetRect(curRect.left, curRect.top  - (size.cy + 3),
						curRect.left + size.cx + 5, curRect.top - 1);
		it->m_numRect = curRect;
		
		if (it->m_bVisible)
		{
			//pDC->Rectangle(curRect);
			//pDC->TextOut(curRect.left + 2, curRect.top + 1, MakeLongNum(it->m_nID));
			pMemDC->Rectangle(0, 0, nWidth, nHeight);
			pMemDC->TextOut(2, 1, MakeLongNum(it->m_nID));
			pDC->BitBlt(curRect.left, curRect.top, nWidth, nHeight, m_pMemDC, 0, 0, SRCCOPY);
		}
		else
		{
		
			//m_pDlg->InvalidateRect(curRect);
			//m_pDlg->UpdateWindow();
		}

		++nNum;
		
		if (it->m_bVisible && !it->m_objects.IsEmpty()) // && bDraw)
			_DrawNumbers(pDC, pMemDC, &it->m_objects, nNum);
		else
			nNum += it->m_objects.Count();

		
		--it;
	}


}
/*----------------------------------------------------------------------*\
	Funkcja    :  CFormCreatorView::DrawInfo
	Wersja     :  1.0
  ----------------------------------------------------------------------
	Opis       :  Rysuje tabelk z opisami obiektw
	Wyjscie    :  void	
	Parametry  :
			      CDC * pDC	-	kontekst urzdzenia rysujcego
	Note       :
\*----------------------------------------------------------------------*/
int CFormCreatorView::JustifyString(CDC * pDC, int nMaxStr, int nColWidth)
{
	int cStr = 0;
	int nSpaceSize = pDC->GetTextExtent(' ').cx;

	int nLines = 0;

	while (cStr < nMaxStr)
	{
		int nSubStrSize = 0;
		int nMaxSubStr = 0;

		int cLstStr = cStr;

		while (cStr < nMaxStr)
		{

			int nTmpStrSize = pDC->GetTextExtent(m_subStr[cStr].m_str).cx;

			nSubStrSize += nTmpStrSize + nSpaceSize;
			
			if (nSubStrSize > nColWidth)
			{
				nSubStrSize -= (nTmpStrSize + nSpaceSize);
				break;
			}

			nMaxSubStr++;	
			cStr++;
		}

		if (nMaxSubStr == 0)
			break;
		
		float dx;

		if (cStr != nMaxStr && nMaxStr > 1)
			dx = (nColWidth - nSubStrSize + (nMaxSubStr - 1) * nSpaceSize) / (float) (nMaxSubStr - 1);
		else 
			dx = nSpaceSize;

		float fPosX = 0;

		for (int i = 0; i < nMaxSubStr; i++)
		{
			float fStrLen = (float) pDC->GetTextExtent(m_subStr[i + cLstStr].m_str).cx;
			m_subStr[i + cLstStr].nPosX = (int) fPosX;
			fPosX += fStrLen + dx;
		}

		nLines++;
	}

	return nLines;
}
int CFormCreatorView::SubstractString(CString  & str)
{
	int nStrLen = str.GetLength();

	int i = 0;
	int j = 0;

	/* pominicie spacji na pocztku tekstu */

	while (i < nStrLen && str[i] == ' ') i++;
	
	int nLstPos = i;

	while (i < nStrLen)
	{
		if (str[i] == ' ')
		{

			m_subStr[j].m_str = str.Mid(nLstPos, i - nLstPos);

			while (i < nStrLen && str[i] == ' ') i++;
			
			nLstPos = i;

			j++;
		}
		i++;
	}
	m_subStr[j].m_str = str.Mid(nLstPos, nStrLen - nLstPos);

	return j + 1;
}
void CFormCreatorView::_DrawInfo(CDC * pDC, float & fPosY, int & cNum, int nCountColLeft, int nNameColLeft, int nInfoColLeft, int nInfoColWidth, CTList<CTObject *> * objects)
{
	CString strNum;
	objIterator it = objects->last();

	const int nLineHeight = 18;

	while (it != NULL)
	{
		int nMaxStr = SubstractString(it->m_strInfo);
		int nLines  = JustifyString(pDC, nMaxStr, nInfoColWidth);
		
		int nCellStartY =  fPosY - 6;
		int nCellEndY   =  fPosY + nLines * nLineHeight + 5;
		pDC->Rectangle(m_nTabLeft, nCellStartY, m_nTabRight, nCellEndY);

		pDC->MoveTo(nNameColLeft - 5, nCellStartY);
		pDC->LineTo(nNameColLeft - 5, nCellEndY);

		pDC->MoveTo(nInfoColLeft - 5, nCellStartY);
		pDC->LineTo(nInfoColLeft - 5, nCellEndY);
		
		strNum.Format("%d", cNum);
	
		pDC->TextOut(nCountColLeft, (int) fPosY, strNum);
		pDC->TextOut(nNameColLeft,  (int) fPosY, it->m_strName);



		for (int i = 0; i < nMaxStr; i++)
		{
			float fPosX = nInfoColLeft + m_subStr[i].nPosX;

			if (i > 0 && m_subStr[i].nPosX == 0)
				fPosY += 18;

			pDC->TextOut((int) fPosX, (int) fPosY, m_subStr[i].m_str);
		}

		++cNum;

		fPosY += nLineHeight + 5;
		
		_DrawInfo(pDC, fPosY, cNum, nCountColLeft, nNameColLeft, nInfoColLeft, nInfoColWidth, &it->m_objects);

		--it;
	}
}
void CFormCreatorView::DrawInfo(CDC * pDC)
{
	pDC->SetMapMode(MM_LOENGLISH);

	/* przesunicie kartki wzgldem widoku */
	const int   ciSpaceX = 15;
	const int   ciSpaceY = 15;

	/* wymiary kartki przeskalowane na ekran monitora */

	const float cfRealScale = 3.19f;
	const float cfPageSizeX = 210 * cfRealScale;
	const float cfPageSizeY = 297 * cfRealScale;

	CRect pgnRect(ciSpaceX, ciSpaceY, ciSpaceX + cfPageSizeX, ciSpaceY + cfPageSizeY);

	CRect  dlgRect   = GetDlgScreenRect();
	CSize  totalSize = GetTotalSize();
	CPoint scrollPos = GetScrollPosition();

	m_dlgRect = dlgRect;

	dlgRect.bottom += scrollPos.y;

	int nLeft   = ciSpaceX * 2;
	int nRight  = cfPageSizeX;
	int nTop    = dlgRect.bottom + ciSpaceY;

	m_nTabLeft  = nLeft;
	m_nTabRight = nRight;

	/* obliczenie szerokoci poszczeglnych kolumn */

	int nCountColSizeP = 10;  /* w procentach */
	int nNameColSizeP  = 30;  /* w procentach */
	int nNameColSize   = (nRight - nLeft) * nNameColSizeP  / 100;
	int nCountColSize  = (nRight - nLeft) * nCountColSizeP / 100;
	
	int nRowSize = 30;
	int nMargin = 5;

	int nCountColLeft  = nLeft + nMargin;
	int nCountColRight = nLeft + nCountColSize - nMargin;
	int nNameColLeft   = nCountColRight + nMargin * 2;
	int nNameColRight  = nNameColLeft + nNameColSize - nMargin * 2;
	int nInfoColLeft   = nNameColRight + nMargin * 2;
	int nInfoColRight  = nRight - nMargin;

	int nRowBottom = nTop + nRowSize;

	int nNameColWidth  = nNameColRight - nNameColLeft;
	int nInfoColWidth  = nInfoColRight - nInfoColLeft;
	int nCountColWidth = nCountColRight - nCountColLeft;

	int nInfoHeight = nRowSize;


	int nBottom = dlgRect.bottom + ciSpaceY + nInfoHeight;

	if (nBottom < nRowBottom)
		nBottom = nRowBottom;



	CPen boldLine(PS_SOLID, 2, RGB(0,0,0));
	CPen thinLine(PS_SOLID, 1, RGB(0,0,0));
	
	pDC->SelectObject(&fontNormal);

	pDC->Rectangle(nLeft, nTop, nRight, nBottom);

	CRect clnRect;
	GetClientRect(&clnRect);
	pDC->FillSolidRect(nLeft - 0, nBottom, nRight - nLeft + 0, clnRect.bottom, RGB(162,173,187));
	
	pDC->MoveTo(nLeft + nCountColSize, nTop);
	pDC->LineTo(nLeft + nCountColSize, nBottom - 0);
	pDC->MoveTo(nLeft + nNameColSize + nCountColSize, nTop);
	pDC->LineTo(nLeft + nNameColSize + nCountColSize, nBottom - 0);

	pDC->SelectObject(&fontBold);
	pDC->SetBkColor(RGB(255,255,255));
	pDC->SetTextAlign(TA_LEFT | TA_TOP);

	CSize textSize = pDC->GetTextExtent("Lp");

	pDC->TextOut((nCountColLeft + nCountColRight) / 2 - textSize.cx / 2,
		          nTop + nRowSize / 2 - textSize.cy / 2,
				 "Lp");

	textSize = pDC->GetTextExtent("Nazwa");

	pDC->TextOut((nNameColLeft + nNameColRight) / 2  - textSize.cx / 2, 
		         nTop  + nRowSize / 2  - textSize.cy / 2,
				 "Nazwa");
	
	textSize = pDC->GetTextExtent("Opis");

	pDC->TextOut((nInfoColLeft + nInfoColRight) / 2  - textSize.cx / 2, 
		         nTop  + nRowSize / 2  - textSize.cy / 2,
				 "Opis");


	pDC->SelectObject(&fontNormal);
	pDC->SelectObject(&thinLine);
		

	/* Wpisanie informacji do tabelki. Informacja o obiekcie jest rozcigana
	   na ca szeroko kolumny */


	float fPosY = nTop + nRowSize + nMargin;

	int cNum = 1;

	_DrawInfo(pDC, fPosY, cNum, nCountColLeft, nNameColLeft, nInfoColLeft, nInfoColWidth, &m_objects);

	totalSize = GetTotalSize();

	int nMaxX = totalSize.cx;
	int nMaxY = fPosY + ciSpaceY;

	if (nBottom + ciSpaceY > nMaxY)
		nMaxY = nBottom + ciSpaceY;
	if (nRight + ciSpaceX > nMaxX)
		nMaxX = nRight + ciSpaceX;
	if (dlgRect.Width() + dlgRect.left > nMaxX)
		nMaxX = dlgRect.Width() + ciSpaceX + dlgRect.left;
	if (dlgRect.Height() + dlgRect.top> nMaxY)
		nMaxY = dlgRect.Height() + ciSpaceY + dlgRect.top;

	/* jeli tabelka jest za dua i nie mieci si w widoku, lub
	   jeli formatka jest jeszcze wiksza, to tu nastpuje dopasowanie
	   wymiarw okna */

	SetScrollSizes(MM_TEXT, CSize(nMaxX, nMaxY));
	if (m_bDlgSelected)
		UpdateObjectPosInfo(GetDlgScreenRect());

	m_dlgRect = GetDlgScreenRect();

}

void CFormCreatorView::DrawTrackerRect()
{
	m_pTracker->SetMask(mskNothing);

	CDC * pDC;

	/* jesli to formatka */
	if (m_bDlgSelected)
	{
		pDC = GetDC();
		m_pTracker->SetMask(mskMove      | mskTopLeft | mskTopRight | mskBottomLeft |
			                mskMiddleTop | mskMiddleLeft);
		m_pTracker->DrawTrackerRect(pDC, GetDlgScreenRect(), false);
		ReleaseDC(pDC);
	}
	/* lub jeli to obiekty */
	else
	{
		if (m_nObjSelected == 1)
		{
		
			switch (m_pTObject->m_nType)
			{
				case CTRL_COMBOBOX:
					m_pTracker->SetMask(mskWidth);
					break;
				case  CTRL_MENU:
				case  CTRL_POPUPMENU:
					m_pTracker->SetMask(mskSize);
					break;
			}
		}
	
		pDC = m_pDlg->GetDC();
		m_pTracker->DrawMultiTrackerRect(pDC);
		m_pDlg->ReleaseDC(pDC);

	}
	DrawNumbers();

}
/*----------------------------------------------------------------------*\
	Funkcja    :  CFormCreatorView::DrawA4Page
	Wersja     :  1.0
  ----------------------------------------------------------------------
	Opis       :  Rysuje kontury strony A4
	Wyjscie    :  void
	Parametry  :
			      CDC *pDC	-	kontekst urzdzenia rysujcego
	Note       :
\*----------------------------------------------------------------------*/
void CFormCreatorView::DrawA4Page(CDC *pDC)
{
	const int ciSpaceX = 15;
	const int ciSpaceY = 15;
	const float cfRealScale = 3.19f;
	const float cfPageSizeX = 210 * cfRealScale;
	const float cfPageSizeY = 297 * cfRealScale;


	CSize totalSize = GetTotalSize();

	CRect clnRect;
	GetClientRect(clnRect);

	CRect dlgRect = GetDlgScreenRect();
	CRect drwRect;

	CRect pgnRect(ciSpaceX, ciSpaceY, ciSpaceX + cfPageSizeX, ciSpaceY + cfPageSizeY);
	pDC->FrameRect(pgnRect, NULL);
	pDC->SetBkColor(RGB(162,173,187));
	pDC->SelectObject(&fontNormal);
	pDC->TextOut(ciSpaceX + 3, ciSpaceY + 3, "Page 1");

}


/*----------------------------------------------------------------------*\
	Funkcja    :  CFormCreatorView::EraseDlgDragRect
	Wersja     :  1.0
  ----------------------------------------------------------------------
	Opis       :  Uswa obwdk z obiektu.
	Wyjscie    :  void
	Parametry  :
	Note       :
\*----------------------------------------------------------------------*/

void CFormCreatorView::EraseDlgDragRect()
{

	CRect dlgRect;
	CBrush brush;

	if (!m_pDlg) return;
	
	brush.CreateSolidBrush(RGB(162,173,187));
	m_pDlg->GetWindowRect(&dlgRect);
	ScreenToClient(&dlgRect);

	int nTWidth  = m_pTracker->m_nTrackerWidth;
	int nTHeight = m_pTracker->m_nTrackerHeight;
	int nLeft    = dlgRect.left;
	int nRight   = dlgRect.right;
	int nTop     = dlgRect.top;
	int nBottom  = dlgRect.bottom;

	CDC * pDC = GetDC();
	pDC->FillRect(CRect(nLeft - nTWidth, nTop - nTHeight, nRight + nTWidth, nTop), &brush);
	pDC->FillRect(CRect(nLeft - nTWidth, nBottom, nRight + nTWidth, nBottom + nTHeight), &brush);
	pDC->FillRect(CRect(nLeft - nTWidth, nTop, nLeft, nBottom), &brush);
	pDC->FillRect(CRect(nRight, nTop, nRight + nTWidth, nBottom), &brush);
	ReleaseDC(pDC);

}
