#include "stdafx.h"
#include "FormCreator.h"
#include "Globals.h"

#include "GridCtrl/GridCtrl.h"
#include "Console.h"
#include "FormCreatorDoc.h"
#include "FormCreatorView.h"
#include "MultiRectTracker.h"
#include "ToolbarControls.h"

#include "DialogGrid.h"
#include "DialogTemplate.h"
#include "ColumnsDialog.h"

#include "TObject.h"
#include "PanelControl.h"

#include "TransparentWindow.h"
#include "math.h"
#include <afxrich.h>

#include <iostream>
using namespace std;

bool CFormCreatorView::TMP_AddObject(TFCObjBuffer * pObjBuffer)
{

	CString strParentGuid = pObjBuffer->sObjPrnID;
	CString strObjectGuid = pObjBuffer->sObjID;
	
	CTObject * pTParent;
	CTObject * pTObject;

	pTObject = FindObjectByGuid(&m_objects, strObjectGuid);

	if (pTObject)
		return false;

	if (strParentGuid == "")
		pTParent = m_pTDlg;
	else
	{
		pTParent = FindObjectByGuid(&m_objects, strParentGuid);
		if (!pTParent)
			return false;
	}


	CRect objRect(pObjBuffer->nObjPosX,  pObjBuffer->nObjPosY,
                  pObjBuffer->nObjPosX + pObjBuffer->nObjWidth,
				  pObjBuffer->nObjPosY + pObjBuffer->nObjHeight);

	
	int nType = GetObjectType(CString(pObjBuffer->sObjType));

	if (nType < 0) return true;

	int nClass = GetObjectClass(CString(pObjBuffer->sObjType));

	if (cCtrlInfo[nType].pClassType[nClass].nHeight != 0)
		objRect.bottom = objRect.top + DEF_HEIGHT;
	
	AddObject(pTParent, nType, GetObjectRect(pTParent, objRect), true, true, false);
	
	m_pTObject->m_pObject->SetWindowText(pObjBuffer->sObjCaption);
	m_pTObject->m_pObject->ShowWindow(SW_SHOW);
	m_pTObject->m_strGuid = strObjectGuid;
	m_pTObject->m_strName = pObjBuffer->sObjName;
	m_pTObject->m_nClass  = nClass;
	m_pTObject->m_strInfo = pObjBuffer->sObjDescr;


	CString strCaption = pObjBuffer->sObjCaption;
	CString strItem;

	int nStrLen = strCaption.GetLength();

	bool bCaption   = true;
	bool bFirstItem = true;

	int i = 0;

	if (nType == CTRL_PANEL       || nType == CTRL_BUTTON      ||
		nType == CTRL_TEXTBOX     || nType == CTRL_STATICTEXT  ||
		nType == CTRL_GROUPBOX    || nType == CTRL_RADIOBUTTON ||
		nType == CTRL_CHECKBUTTON || nType == CTRL_RICHEDIT)

		m_pTObject->m_strCaption = strCaption;

	else
	{
		m_pTObject->m_strCaption = "";

		while (i < nStrLen)
		{
			char ch = strCaption[i];
			strItem += ch;

			if (ch == '|' || i == nStrLen - 1) 
			{
				if (i != nStrLen - 1)
					strItem.SetAt(strItem.GetLength() - 1, '\0');

				switch (nType)
				{
					case CTRL_GRID:
									if (bFirstItem)
										((CGridCtrl *) m_pTObject->m_pObject)->DeleteColumn(1);								
									((CGridCtrl *) m_pTObject->m_pObject)->InsertColumn(strItem);

									break;
					case CTRL_LISTBOX:
									if (bFirstItem)
										((CListBox *) m_pTObject->m_pObject)->ResetContent();
									((CListBox *) m_pTObject->m_pObject)->AddString(strItem);
									
									break;
					case CTRL_COMBOBOX:
									if (bFirstItem)
										((CComboBox *) m_pTObject->m_pObject)->ResetContent();
									((CComboBox *) m_pTObject->m_pObject)->AddString(strItem);
									break;
				}

				bFirstItem = false;
				strItem = "";
			}

			++i;
		}
	}


	if (strcmp(pObjBuffer->sObjState, "Aktywny") == 0)
		m_pTObject->m_bEnabled = true;
	else
		m_pTObject->m_bEnabled = false;

	m_pTObject->m_pObject->EnableWindow(m_pTObject->m_bEnabled);

	AddToObjList(m_pTObject);
	AddToInfObjList(m_pTObject);

	return true;

}

void CFormCreatorView::TMP_LoadObjects(CFile & file)
{

	TFCHeaderBuffer hdrBuffer;
	file.Read(&hdrBuffer, sizeof(TFCHeaderBuffer));

	if (hdrBuffer.nWidth != -1)
		m_pDlg->MoveWindow(20, 20, hdrBuffer.nWidth, hdrBuffer.nHeight, true);

	m_strGuid = hdrBuffer.sStateID;

	TFCObjBuffer * pObjBufferTab = new TFCObjBuffer[hdrBuffer.nObjCount];
	TFCObjBuffer * pObjBuffer = pObjBufferTab;


	int * pObjBufferFlag = new int[hdrBuffer.nObjCount];


	int nObj = 0;

	while (nObj < hdrBuffer.nObjCount)
	{
		file.Read(pObjBuffer, sizeof(TFCObjBuffer));


		if (pObjBuffer->nObjWidth == 0)
			pObjBuffer->nObjWidth = 40;
		if (pObjBuffer->nObjHeight == 0)
			pObjBuffer->nObjHeight = 18;

		m_objBuffer.InsFront(pObjBuffer);

		pObjBufferFlag[nObj] = 0;

		pObjBuffer++;	

		nObj++;
	}

	nObj = 0;

	int i = 0;

	while (nObj < hdrBuffer.nObjCount)
	{

		if (!pObjBufferFlag[i] && TMP_AddObject(&pObjBufferTab[i]))
		{
			pObjBufferFlag[i] = 1;
			nObj++;
		}

		i++;
		if (i > hdrBuffer.nObjCount - 1)
			i = 0;
	}

	delete [] pObjBufferTab;
	delete [] pObjBufferFlag;
	
	Invalidate();
}

void CFormCreatorView::TMP_SaveObjects(CFile & file, CTList<CTObject *> * objects)
{
	objIterator it = objects->last();
	
	TFCObjBuffer objBuffer;
		
	while (it != NULL)
	{
		memset(&objBuffer, 0, sizeof(TFCObjBuffer));

		strcpy(objBuffer.sObjID,      (LPCSTR) it->m_strGuid);
		strcpy(objBuffer.sObjPrnID,   (LPCSTR) it->m_pTParent->m_strGuid);
		strcpy(objBuffer.sObjType,    (LPCSTR) cCtrlInfo[it->m_nType].pClassType[it->m_nClass].strClass);
		strcpy(objBuffer.sObjName,    (LPCSTR) it->m_strName);
		strcpy(objBuffer.sObjPrnID,   (LPCSTR) it->m_pTParent->m_strGuid);

		CString strCaption = it->m_strCaption;
		CString strItem;
		int nCount;
		int i;

		int nType = it->m_nType;

		if (nType != CTRL_PANEL       && nType != CTRL_BUTTON      &&
			nType != CTRL_TEXTBOX     && nType != CTRL_STATICTEXT  &&
			nType != CTRL_GROUPBOX    && nType != CTRL_RADIOBUTTON &&
			nType != CTRL_CHECKBUTTON && nType != CTRL_RICHEDIT)
		
			strCaption = "";


		switch (it->m_nType)
		{
			case CTRL_GRID:

							nCount = ((CGridCtrl *) it->m_pObject)->GetColumnCount();

							for (i = 1; i < nCount; i++)
								strCaption += ((CGridCtrl *) it->m_pObject)->GetItemText(0, i) + "|";				
							strCaption.SetAt(strCaption.GetLength() - 1, '\0');

							
							break;
			case CTRL_LISTBOX:

							nCount = ((CListBox *) it->m_pObject)->GetCount();

							for (i = 0; i < nCount; i++)
							{
								((CListBox *) it->m_pObject)->GetText(i, strItem);
								strCaption += strItem + "|";
							}
							strCaption.SetAt(strCaption.GetLength() - 1, '\0');

							break;

			case CTRL_COMBOBOX:
							
							nCount = ((CComboBox *) it->m_pObject)->GetCount();

							for (i = 0; i < nCount; i++)
							{
								((CComboBox *) it->m_pObject)->GetLBText(i, strItem);
								strCaption += strItem + "|";
							}
				
							strCaption.SetAt(strCaption.GetLength() - 1, '\0');

							break;
		}


		strcpy(objBuffer.sObjCaption, (LPCSTR) strCaption);
		it->m_strInfo = m_pGrdInfObjList->GetItemText(FindRowInObjInfList(it->m_strName), 2);
		strcpy(objBuffer.sObjDescr, (LPCSTR) it->m_strInfo);

		if (it->m_bEnabled)
			strcpy(objBuffer.sObjState, "Aktywny");
		else
			strcpy(objBuffer.sObjState, "Nieaktywny");

		CRect objRect = GetObjectRealRect(*it, it->m_curRect);

		objBuffer.nObjPosX   = objRect.left;
		objBuffer.nObjPosY   = objRect.top;
		objBuffer.nObjWidth  = objRect.Width();
		objBuffer.nObjHeight = objRect.Height();

		file.Write(&objBuffer, sizeof(TFCObjBuffer));
		
		TMP_SaveObjects(file, &it->m_objects);

		--it;

	}
}
