#include "stdafx.h"
#include "FormCreator.h"
#include "Globals.h"

#include "GridCtrl/GridCtrl.h"
#include "Console.h"
#include "FormCreatorDoc.h"
#include "FormCreatorView.h"
#include "MultiRectTracker.h"
#include "ToolbarControls.h"

#include "DialogGrid.h"
#include "DialogTemplate.h"
#include "DialogPanelControl.h"
#include "ColumnsDialog.h"

#include "TObject.h"

#include "PanelControl.h"

#include "TransparentWindow.h"
#include "math.h"
#include "Guid.h"
#include <afxrich.h>

#include <iostream>
using namespace std;

/*-------------------------------------------------------------------------------------------*\
                                           MENU
\*-------------------------------------------------------------------------------------------*/

void CFormCreatorView::ChangeGridSize(int nGridX, int nGridY)
{
	
	m_nGridX = nGridX;
	m_nGridY = nGridY;

	if (m_bSnapToGrid)
		m_pTracker->SetGrid(m_nGridX, m_nGridY);

	if (m_bShowGrid)
	{
	
		m_pDlg->Invalidate(true);
		m_pDlg->UpdateWindow();

		DrawTrackerRect();
	}

}
void CFormCreatorView::OnGrid0() 
{
	OnTbSgrid();
}

void CFormCreatorView::OnGrid1() 
{
	ChangeGridSize(5, 5);
}

void CFormCreatorView::OnGrid2() 
{
	ChangeGridSize(8, 8);
}

void CFormCreatorView::OnGrid3() 
{
	ChangeGridSize(10, 10);
}

void CFormCreatorView::OnGrid4() 
{
	ChangeGridSize(15, 15);
}
void CFormCreatorView::OnGridUser()
{
	CGridDialog dlg;

	dlg.m_nGridX = m_nGridX;
	dlg.m_nGridY = m_nGridY;
	
	int iRes = dlg.DoModal();

	if (iRes == IDOK)
		ChangeGridSize(dlg.m_nGridX, dlg.m_nGridY);
}

void CFormCreatorView::OnUpdateGrid0(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_bShowGrid);
}

void CFormCreatorView::OnUpdateGrid1(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_nGridX == 5 && m_nGridX == 5);
}

void CFormCreatorView::OnUpdateGrid2(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_nGridX == 8 && m_nGridX == 8);
}

void CFormCreatorView::OnUpdateGrid3(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_nGridX == 10 && m_nGridX == 10);
}

void CFormCreatorView::OnUpdateGrid4(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_nGridX == 15 && m_nGridX == 15);	
}

void CFormCreatorView::OnUpdateGridUser(CCmdUI* pCmdUI) 
{

	pCmdUI->SetCheck(!(m_nGridX == m_nGridY && 
	                  (m_nGridX == 5  || 
		               m_nGridX == 8  ||
		               m_nGridX == 10 ||
		               m_nGridX == 15)));
	
}
void CFormCreatorView::OnEditCopy() 
{
	CTList<CTObject *>::iterator it = m_pTracker->m_selObjects.begin();
	
	m_selObjects.RemoveAll();

	while (it != NULL)
	{
		m_selObjects.InsBack(*it);
		++it;
	}
}

void CFormCreatorView::AddChildObjects(CTObject * pTPParent, CTObject * pTParent, CTList<CTObject *> * objects, int nDepth)
{

	objIterator it = objects->begin();

	if (it == NULL)
		return;

	while (it != NULL)
	{
			
		if (*it != pTPParent)
		{
		
			CopyObject(pTParent, *it, it->m_curRect);

			AddToObjList(m_pTObject);
			AddToInfObjList(m_pTObject);
			AddChildObjects(pTPParent, m_pTObject, &it->m_objects, nDepth + 1);
		}

		++it;
	}

}
void CFormCreatorView::OnEditPaste() 
{
	CTList<CTObject *>::iterator it = m_selObjects.begin();

	if (it == NULL)
		return;

	int nMinX = it->m_curRect.left;
	int nMaxX = it->m_curRect.right;
	int nMinY = it->m_curRect.top;
	int nMaxY = it->m_curRect.bottom;

	bool bDestIsSelected = false;
	while (it != NULL)
	{
		if (*it == m_pTObject)
		{
			bDestIsSelected = true;
			//break;
		}
		
		if (it->m_curRect.left   < nMinX) nMinX = it->m_curRect.left;
		if (it->m_curRect.right  > nMaxX) nMaxX = it->m_curRect.right;
		if (it->m_curRect.top    < nMinY) nMinY = it->m_curRect.top;
		if (it->m_curRect.bottom > nMaxY) nMaxY = it->m_curRect.bottom;


		++it;
	}

	int nMidX = (nMaxX + nMinX) / 2;
	int nMidY = (nMaxY + nMinY) / 2;
	
	CTObject * pTTmpObj = (*m_selObjects.begin())->m_pTParent;
	if (m_nObjSelected == 1 &&
		(m_pTObject->m_nType == CTRL_TABCONTROL ||
		 m_pTObject->m_nType == CTRL_PANEL))
		 pTTmpObj = m_pTObject;

	int nPMidX = pTTmpObj->m_curRect.Width() / 2;
	int nPMidY = pTTmpObj->m_curRect.Height() / 2;

	m_pTracker->UpdateSelObjectsBkg(m_pDlg, UPD_CURPOS);
	m_pTracker->RemoveAllObjects();
	
	it = m_selObjects.begin();

	CTObject *pCloneTObject;
	while (it != NULL)
	{
		CRect rect = it->m_curRect;
		if (pTTmpObj == it->m_pTParent)
		{
			rect.left   += 10;
			rect.right  += 10;
			rect.top    += 10;
			rect.bottom += 10;
		}
		else
		{
			int nWidth = rect.Width();
			int nHeight = rect.Height();

			rect.left = nPMidX - (nMidX - rect.left);
			rect.top  = nPMidY - (nMidY - rect.top);

			rect.right = rect.left + nWidth;
			rect.bottom = rect.top + nHeight;

		}

	
		CopyObject(pTTmpObj, *it, rect);
		AddToObjList(m_pTObject);
		AddToInfObjList(m_pTObject);

//		CreateFlatObjects(&it->m_objects);
		//m_pTObject->m_pObject->ShowWindow(SW_SHOW);
		AddChildObjects(m_pTObject, m_pTObject, &it->m_objects, 0);
		++it;
	}
	ShowAllObjects(-1, &m_objects);
//	m_pDlg->InvalidateRect(rect, false);
	m_pDlg->UpdateWindow();

	m_bDlgSelected = false;
	m_pTracker->m_bDlgSelected = false;

	m_nObjSelected = m_pTracker->m_nObjSelected;

	DrawTrackerRect();

	m_pTObject = *m_pTracker->m_selObjects.begin();
	m_pLstTObject = m_pTObject;
	m_selObjects.RemoveAll();
	
}

void CFormCreatorView::OnEditCut() 
{
	OnKeyDown(VK_DELETE, 0, 0);
}

void CFormCreatorView::OnUpdateEditCopy(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_pTracker->m_selObjects.IsEmpty());
}

void CFormCreatorView::OnUpdateEditPaste(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_selObjects.IsEmpty());
}

void CFormCreatorView::OnUpdateEditCut(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(!m_selObjects.IsEmpty());
}
