#include "stdafx.h"
#include "MultiRectTracker.h"
#include "math.h"
#include "Globals.h"
#include "Console.h"
#include "TObject.h"

#include <iostream>
using namespace std;

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

inline int _abs(int x)
{
	return x < 0 ? -x: x;
}

struct MousePos
{
	int     iBit[4];  /* numery warunkw dla danego obszaru */
	bool    iChg[4];  /* flagi kierunku myszki */
	HCURSOR hCur;     /* numer kursora w resourcach */
	int     nMsk;     /* numer maski */
	int     iRet;     /* warto okrelajca pooenie myszki w prostokcie */
};

const int iLeftMax      = 0;
const int iLeftMin      = 1;
const int iRightMax     = 2;
const int iRightMin     = 3;
const int iTopMax       = 4;
const int iTopMin       = 5;
const int iBottomMax    = 6;
const int iBottomMin    = 7;
const int iHeightMidMin = 8;
const int iHeightMidMax = 9;
const int iWidthMidMin  = 10;
const int iWidthMidMax  = 11;

const int piMaxMousePos = 12;
const int ciMaxMousePos = 13;

/* Tablica flag okrelajcych, czy dany punkt spenia warunek wzgldem okrelonej pozycji,
   np. jeli mouse.x < rect.left to piMousePos[iLeftMin] = 1, itd. */

int piMousePos[piMaxMousePos]; 

MousePos ciMousePos[ciMaxMousePos] =                      /* left,  top,   right, bottom */
{
	iLeftMax   , iLeftMin   , iTopMax       , iTopMin,       true,  true,  false, false, NULL,  mskTopLeft,      hitTopLeft,
	iLeftMax   , iLeftMin   , iBottomMax    , iBottomMin,    true,  false, false, true,  NULL,  mskBottomLeft,   hitBottomLeft,
	iRightMax  , iRightMin  , iTopMax       , iTopMin,       false, true,  true,  false, NULL,  mskTopRight,     hitTopRight,
	iRightMax  , iRightMin  , iBottomMax    , iBottomMin,    false, false, true,  true,  NULL,  mskBottomRight,  hitBottomRight,

	iLeftMax   , iLeftMin   , iHeightMidMax , iHeightMidMin, true,  false, false, false, NULL,  mskMiddleLeft,   hitMiddleLeft,
	iRightMax  , iRightMin  , iHeightMidMax , iHeightMidMin, false, false, true,  false, NULL,  mskMiddleRight,  hitMiddleRight,
	iTopMax    , iTopMin    , iWidthMidMin  , iWidthMidMax,  false, true,  false, false, NULL,  mskMiddleTop,    hitMiddleTop,
	iBottomMax , iBottomMin , iWidthMidMin  , iWidthMidMax,  false, false, false, true,  NULL,  mskMiddleBottom, hitMiddleBottom,

	iLeftMax   , iRightMax  , iTopMax       , iTopMin,       true,  true,  true,  true,  NULL,  mskTop,          hitTop,
	iLeftMax   , iRightMax  , iBottomMax    , iBottomMin,    true,  true,  true,  true,  NULL,  mskBottom,       hitBottom,
	iTopMax    , iBottomMax , iLeftMin      , iLeftMax,      true,  true,  true,  true,  NULL,  mskLeft,         hitLeft,
	iTopMax    , iBottomMax , iRightMin     , iRightMax,     true,  true,  true,  true,  NULL,  mskRight,        hitRight,

	iLeftMax   , iRightMax  , iTopMax       , iBottomMax,    true,  true,  true,  true,  NULL,  mskMiddle,       hitMiddle
};



CMultiRectTracker::CMultiRectTracker()
{
	m_nTrackerWidth = 6;
	m_nTrackerHeight = 6;

	m_nGridX = 1;
	m_nGridY = 1;

	m_nObjSelected = 0;

	CWinApp * pWinApp = AfxGetApp();

	m_hcurArrow = pWinApp->LoadCursor(IDC_CURSOR_ARROW);
	m_hcurMove  = pWinApp->LoadCursor(IDC_CURSOR_MOVE);
	m_hcurSize1 = pWinApp->LoadCursor(IDC_CURSOR_SIZE1);
	m_hcurSize2 = pWinApp->LoadCursor(IDC_CURSOR_SIZE2);
	m_hcurSize3 = pWinApp->LoadCursor(IDC_CURSOR_SIZE3);
	m_hcurSize4 = pWinApp->LoadCursor(IDC_CURSOR_SIZE4);

	ciMousePos[hitLeft].hCur         = m_hcurMove;
	ciMousePos[hitRight].hCur        = m_hcurMove;
	ciMousePos[hitTop].hCur          = m_hcurMove;
	ciMousePos[hitBottom].hCur       = m_hcurMove;
	ciMousePos[hitMiddle].hCur       = m_hcurMove;
	ciMousePos[hitTopLeft].hCur      = m_hcurSize2;
	ciMousePos[hitTopRight].hCur     = m_hcurSize1;
	ciMousePos[hitBottomLeft].hCur   = m_hcurSize1;
	ciMousePos[hitBottomRight].hCur  = m_hcurSize2;
	ciMousePos[hitMiddleLeft].hCur   = m_hcurSize3;
	ciMousePos[hitMiddleRight].hCur  = m_hcurSize3;
	ciMousePos[hitMiddleTop].hCur    = m_hcurSize4;
	ciMousePos[hitMiddleBottom].hCur = m_hcurSize4;

	StoreObjectsPrevPos();

	m_nMask = mskNothing;

	m_bDragNDrop = false;
}

CMultiRectTracker::~CMultiRectTracker()
{
}


CBrush * CMultiRectTracker::LoadDIBBrush(int IDB_BRUSH)
{
	HRSRC hRes; 
	HGLOBAL hData, hLockedData;

	CBrush * brush;

	hRes = ::FindResource(AfxGetResourceHandle(), MAKEINTRESOURCE(IDB_BRUSH), RT_BITMAP);
	if (hRes != NULL)
	{
	  if (((hData = ::LoadResource(AfxGetResourceHandle(), hRes)) != NULL) && 
		  ((hLockedData = (HGLOBAL)::LockResource(hData)) != NULL))
	  {
		    brush = new CBrush;
			brush->CreateDIBPatternBrush(hLockedData, DIB_RGB_COLORS);

			::FreeResource(hLockedData);
	  }
	}

	return brush;

}

void CMultiRectTracker::SetRect(CRect & rect)
{
	m_curRect = rect;
}

void CMultiRectTracker::QuantizeRect(CRect & rect, bool bCurOld)
{
	int dxL, dyT;
	int dxR, dyB;

	CRect chgRect;

	if (bCurOld)
		chgRect = m_curRect;
	else
		chgRect = m_tmpRect;


	dxL = Nearest(chgRect.left,    m_nGridX) - chgRect.left; 
	dxR = Nearest(chgRect.right,   m_nGridX) - chgRect.right; 
	dyT = Nearest(chgRect.top,     m_nGridY) - chgRect.top;
	dyB = Nearest(chgRect.bottom,  m_nGridY) - chgRect.bottom;

	if (chgRect.left   + dxL < m_clpRect.left)   dxL += m_nGridX; else
	if (chgRect.left   + dxL > m_clpRect.right)  dxL -= m_nGridX;

	if (chgRect.right  + dxR > m_clpRect.right)  dxR -= m_nGridX; else
	if (chgRect.right  + dxR < m_clpRect.left)   dxR += m_nGridX;

	if (chgRect.top    + dyT < m_clpRect.top)    dyT += m_nGridY; else
	if (chgRect.top    + dyT > m_clpRect.bottom) dyT -= m_nGridY;

	if (chgRect.bottom + dyB > m_clpRect.bottom) dyB -= m_nGridY; else
	if (chgRect.bottom + dyB < m_clpRect.top)    dyB += m_nGridY;

	
	switch (m_nHitTest)
	{
		case hitMiddleLeft:
				rect.left   += dxL;
				break;
		case hitMiddleRight:
				rect.right  += dxR;
				break;
		case hitMiddleTop:
				rect.top    += dyT;
				break;
		case hitMiddleBottom:
				rect.bottom += dyB;
				break;
		case hitTopRight:
				rect.top    += dyT;
				rect.right  += dxR;
				break;
		case hitTopLeft:
				rect.top    += dyT;
				rect.left   += dxL;
				break;
		case hitBottomLeft:
				rect.bottom += dyB;
				rect.left   += dxL;
				break;
		case hitBottomRight:
				rect.bottom += dyB;
				rect.right  += dxR;
				break;
		default:
				
				rect.left   += dxL;
				rect.top    += dyT;
				rect.right  += dxL;
				rect.bottom += dyT;

				if (chgRect.right  + dxL > m_clpRect.right)
				{
					rect.right  -= m_nGridX;
					rect.left   -= m_nGridX;

				}
				if (chgRect.bottom + dyT > m_clpRect.bottom)
				{
					rect.top    -= m_nGridY;
					rect.bottom -= m_nGridY;
				}
			
				break;

	}

	m_qCurRect = rect;
}

void CMultiRectTracker::QuantizeObjectsPos()
{
	CTList<CTObject *>::iterator it = m_selObjects.begin();


	while (it != NULL)
	{
		QuantizeRect(it->m_curRect, true);
		QuantizeRect(it->m_tmpRect, false);
			
		++it;
	}
}
void CMultiRectTracker::InvalidateNumRects(CTList<CTObject *> * objects, CRect * tltRect)
{
	CTList<CTObject *>::iterator it = objects->begin();
	

	while (it != NULL)
	{
		InvalidateNumRects(&it->m_objects, tltRect);

		if (!tltRect)
			m_pParent->InvalidateRect(it->m_numRect);
		else
			ExpandRect(*tltRect, it->m_numRect);
		
		++it;
	}

}
CRect CMultiRectTracker::GetObjectTotalRect(CTList<CTObject *> * objects, CRect & tltRect)
{
	CTList<CTObject *>::iterator it = objects->begin();

	while (it != NULL)
	{
		GetObjectTotalRect(&it->m_objects, tltRect);
		
		ExpandRect(tltRect, it->m_numRect);

		++it;
	}

	return tltRect;
	
}

CRect CMultiRectTracker::GetObjectRect(CTObject * pTObject, CRect & rect)
{
	CRect curRect = rect;

	CTObject * pTCurObject = pTObject;

	while (pTCurObject->m_pObject != m_pParent)
	{
		int nLeft = pTCurObject->m_curRect.left + pTCurObject->m_nBorderLeft;
		int nTop  = pTCurObject->m_curRect.top  + pTCurObject->m_nBorderTop;

		curRect.left   -= nLeft;
		curRect.right  -= nLeft;
		curRect.top    -= nTop;
		curRect.bottom -= nTop;

		pTCurObject = pTCurObject->m_pTParent;
	}

	return curRect;

}

CRect CMultiRectTracker::GetObjectRealRect(CTObject * pTObject, CRect & rect)
{
	CRect curRect = rect;

	CTObject * pTCurObject = pTObject->m_pTParent;

	while (pTCurObject->m_pObject != m_pParent)
	{
		int nLeft = pTCurObject->m_curRect.left + pTCurObject->m_nBorderLeft;
		int nTop  = pTCurObject->m_curRect.top  + pTCurObject->m_nBorderTop;

		curRect.left   += nLeft;
		curRect.right  += nLeft;
		curRect.top    += nTop;
		curRect.bottom += nTop;

		pTCurObject = pTCurObject->m_pTParent;
	}

	return curRect;
}

void CMultiRectTracker::DrawMultiSelectionRect(CDC* pDC, CTList<CTObject *> * objects, bool bDrawOnly)
{

	CTList<CTObject *>::iterator it = objects->begin();

	CRect curRect;
	CRect tmpRect;
	CSize size(1,1);

	while (it != NULL)
	{
		bool bDraw = true;

		if (it->m_pTParent->m_nType == CTRL_TABCONTROL)
		{
			int nPage = ((CTabCtrl *) it->m_pTParent->m_pObject)->GetCurSel();
			if (it->m_nPage != nPage)
				bDraw = false;
		}

		if (bDraw)
		{
		
			curRect = GetObjectRealRect(*it, it->m_curRect);
			QuantizeRect(curRect, true);
			curRect.NormalizeRect();


			if (bDrawOnly)
				pDC->DrawDragRect(curRect, size, NULL, size);
			else
				pDC->DrawDragRect(curRect, size, it->m_tmpRect, size);


			/* test m_nHitTest po to aby nie rysowa zagniedonych obiektw
			   podczas rozcigania, a tylko podczas przeswania */

			if (it->m_objects.begin() != NULL && m_nHitTest >= hitTop)
				DrawMultiSelectionRect(pDC, &it->m_objects, bDrawOnly);

			it->m_tmpRect = curRect;

		}

		++it;
	}

}

void CMultiRectTracker::DrawSelectionRect(CDC* pDC, bool bDrawOnly)
{
	CSize size(1,1);

	CRect cRect(m_curRect);
	CRect oRect(m_tmpRect);

	QuantizeRect(cRect, true);
	cRect.NormalizeRect();
	QuantizeRect(oRect, false);
	oRect.NormalizeRect();
		
	if (bDrawOnly)
		pDC->DrawDragRect(cRect, size, NULL, size);
	else
		pDC->DrawDragRect(cRect, size, oRect, size);

}

void CMultiRectTracker::DrawMultiTrackerRect(CDC * pDC)
{
	CTList<CTObject *>::iterator it = m_selObjects.begin();

	while (it != NULL)
	{
		CRect curRect = CalcDragRect(GetObjectRealRect(*it, it->m_curRect));
	
		if (it != m_selObjects.last() || it == m_selObjects.begin())
			DrawTrackerRect(pDC, curRect, false);
		else
			DrawTrackerRect(pDC, curRect, true);
		++it;
	}
}

void CMultiRectTracker::DrawDottedRect(CDC * pDC, int x0, int y0, int width, int height)
{
	int iSizeOne = 4;
	int iSizeTwo = 3;

	if (width > 0)

	for (int i = x0; i <= x0 + width; i += 4)
	{
		
		for (int j = 0; j < iSizeOne; j ++)
			pDC->SetPixelV(i, y0 + j * 2, RGB(0,0,0));
		for (j = 0; j < iSizeTwo; j ++)
			pDC->SetPixelV(i + 2, y0 + j * 2 + 1, RGB(0,0,0));
	}
	
	if (height > 0)

	for (int i = y0; i <= y0 + height; i += 4)
	{
		for (int j = 0; j < iSizeOne; j ++)
			pDC->SetPixelV(x0 + j * 2, i, RGB(0,0,0));
		for (j = 0; j < iSizeTwo; j ++)
			pDC->SetPixelV(x0 + j * 2 + 1, i + 2, RGB(0,0,0));
	}

}
void CMultiRectTracker::DrawTrackerRect(CDC * pDC, CRect rect, bool bIsLast)
{
	CSize size(m_nTrackerWidth, m_nTrackerHeight);
	
	if (m_bDlgSelected)
		rect = CalcDragRect(rect);
		
	int nLeft     = rect.left;
	int nRight    = rect.right;
	int nTop      = rect.top;
	int nBottom   = rect.bottom;
	int nTHeight  = m_nTrackerHeight;
	int nTWidth   = m_nTrackerWidth;
	int nWMid     = (nRight + nLeft) / 2;
	int nHMid     = (nBottom + nTop) / 2;
	int nTHeightH = nTHeight / 2;
	int nTWidthH  = nTWidth / 2;

	int colMsk;
	int colDrg;
	
	colMsk = RGB(0,128,192);
	if (bIsLast) colDrg = RGB(255,0,0); else 
		         colDrg = RGB(255,255,0);
	

	CPen pen;
	pen.CreatePen(PS_DOT, 1, RGB(150,0,255));
	CPen * oldPen = pDC->SelectObject(&pen);
  
	
	pDC->MoveTo(nLeft  + nTWidthH, nTop    + nTHeightH);
	pDC->LineTo(nRight - nTWidthH, nTop    + nTHeightH);
	pDC->LineTo(nRight - nTWidthH, nBottom - nTHeightH);
	pDC->LineTo(nLeft  + nTWidthH, nBottom - nTHeightH);
	pDC->LineTo(nLeft  + nTWidthH, nTop    + nTHeightH);
		

	if ((m_nMask & mskTopLeft)      == 0) pDC->FillSolidRect(nLeft, nTop, nTWidth, nTHeight, colDrg); else
		                                  pDC->FillSolidRect(nLeft, nTop, nTWidth, nTHeight, colMsk);
		
	if ((m_nMask & mskTopRight)     == 0) pDC->FillSolidRect(nRight - nTWidth, nTop, nTWidth, nTHeight, colDrg); else
	                                      pDC->FillSolidRect(nRight - nTWidth, nTop, nTWidth, nTHeight, colMsk);
		
	if ((m_nMask & mskMiddleTop)    == 0) pDC->FillSolidRect(nWMid - nTWidthH, nTop, nTWidth, nTHeight, colDrg); else
		                                  pDC->FillSolidRect(nWMid - nTWidthH, nTop, nTWidth, nTHeight, colMsk);

	if ((m_nMask & mskBottomLeft)   == 0) pDC->FillSolidRect(nLeft, nBottom - nTHeight, nTWidth, nTHeight, colDrg); else
		                                  pDC->FillSolidRect(nLeft, nBottom - nTHeight, nTWidth, nTHeight, colMsk);

	if ((m_nMask & mskBottomRight)  == 0) pDC->FillSolidRect(nRight - nTWidth, nBottom - nTHeight, nTWidth, nTHeight, colDrg); else
		                                  pDC->FillSolidRect(nRight - nTWidth, nBottom - nTHeight, nTWidth, nTHeight, colMsk);

	if ((m_nMask & mskMiddleBottom) == 0) pDC->FillSolidRect(nWMid - nTWidthH, nBottom - nTHeight, nTWidth, nTHeight, colDrg); else
		                                  pDC->FillSolidRect(nWMid - nTWidthH, nBottom - nTHeight, nTWidth, nTHeight, colMsk);

	if ((m_nMask & mskMiddleLeft)   == 0) pDC->FillSolidRect(nLeft, nHMid - nTHeightH, nTWidth, nTHeight, colDrg); else
		                                  pDC->FillSolidRect(nLeft, nHMid - nTHeightH, nTWidth, nTHeight, colMsk);

	if ((m_nMask & mskMiddleRight)  == 0) pDC->FillSolidRect(nRight - nTHeight, nHMid - nTHeightH, nTWidth, nTHeight, colDrg);	else
		                                  pDC->FillSolidRect(nRight - nTHeight, nHMid - nTHeightH, nTWidth, nTHeight, colMsk);

}

bool CMultiRectTracker::AddObject(CTObject * pTObject)
{
	if (pTObject == NULL)
		return false;
	
	CTList<CTObject *>::iterator it = m_selObjects.begin();

	bool bAlreadyInList = false;

	while (it != NULL)
	{
		if (*it == pTObject)
		{
			bAlreadyInList = true;
			break;
		}

		++it;
	}


	bool bEmptyList = m_selObjects.IsEmpty();
	bool bSameLevel = true;

	if (!bAlreadyInList)		
	{
		// musi by ta sama hierarchia

		if (!bEmptyList)
			bSameLevel = m_selObjects.begin()->m_pTParent == pTObject->m_pTParent;


		if (bSameLevel)
		{			

			if (m_selObjects.IsEmpty())
				m_curRect = GetObjectRealRect(pTObject, pTObject->m_curRect);
			else
				ExpandRect(GetObjectRealRect(pTObject, pTObject->m_curRect));

			it = m_selObjects.InsBack(pTObject);

			m_nObjSelected++;
		}
		else
			bAlreadyInList = true;

	}

	return !bAlreadyInList;
}

bool CMultiRectTracker::IntersectRect(CRect & rect0, CRect & rect1)
{

	bool bLeft   = false;
	bool bRight  = false;
	bool bTop    = false;
	bool bBottom = false;

	if (rect0.left   >= rect1.left && rect0.left   <= rect1.right)  bLeft   = true;
	if (rect0.top    >= rect1.top  && rect0.top    <= rect1.bottom) bTop    = true;
	if (rect0.right  >= rect1.left && rect0.right  <= rect1.right)  bRight  = true;
	if (rect0.bottom >= rect1.top  && rect0.bottom <= rect1.bottom) bBottom = true;


	if (bLeft && bTop)     return true;
	if (bLeft && bBottom)  return true;
	if (bRight && bTop)    return true;
	if (bRight && bBottom) return true;

	return false;

}
void CMultiRectTracker::RecalculateSelctionRect()
{
	CTList<CTObject *>::iterator it = m_selObjects.begin();

	
	m_curRect = it->m_curRect;
	
	it++;

	while (it != NULL)
	{
		ExpandRect(it->m_curRect);
		++it;
	}

}
void CMultiRectTracker::MoveSelObjects(CWnd * pWnd)
{

	CTList<CTObject *>::iterator it = m_selObjects.begin();

	CFormCreatorView * pView = (CFormCreatorView *) pWnd->GetParentFrame()->GetActiveView();

	//HDWP hDWP = BeginDeferWindowPos( m_selObjects.Count() );
	//it = m_selObjects.begin();

	while (it != NULL)
	{
		it->m_pObject->MoveWindow(it->m_curRect, false);

		CRect curRect = GetObjectRealRect(*it, it->m_curRect);
		CRect oldRect = GetObjectRealRect(*it, it->m_oldRect);

		if (pView->m_bShowNumbers && it->m_bVisible)
		{
			CRect numRect = it->m_numRect;
			InvalidateNumRects(&it->m_objects, &numRect);
			ExpandRect(oldRect, numRect);
		}


		pWnd->RedrawWindow(oldRect, NULL, RDW_INVALIDATE | RDW_ERASE | RDW_UPDATENOW);
		bool bb = pView->m_bShowGrid;
		pView->m_bShowGrid = false;
		pWnd->RedrawWindow(curRect, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
		pView->m_bShowGrid = bb;


//		DeferWindowPos(hDWP, it->m_pObject->m_hWnd, NULL, curRect.left, curRect.top, curRect.Width(), curRect.Height(), SWP_NOZORDER | SWP_DRAWFRAME);
/*		pWnd->ModifyStyle(0, WS_CLIPCHILDREN);
		pWnd->RedrawWindow(curRect, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_NOCHILDREN);
		pWnd->ModifyStyle(WS_CLIPCHILDREN, 0);*/

		++it;
	}


//	EndDeferWindowPos( hDWP );

}
void CMultiRectTracker::SetGrid(int nGridX, int nGridY)
{
	m_nGridX = nGridX;
	m_nGridY = nGridY;

}
void CMultiRectTracker::RemoveAllObjects()
{
	m_selObjects.RemoveAll();
	m_curRect.SetRectEmpty();
	m_oldRect.SetRectEmpty();

	m_nObjSelected = 0;
}

void CMultiRectTracker::ExpandRect(CRect & rect)
{
	if (rect.left   < m_curRect.left)   m_curRect.left   = rect.left;
	if (rect.right  > m_curRect.right)  m_curRect.right  = rect.right;
	if (rect.top    < m_curRect.top)    m_curRect.top    = rect.top;
	if (rect.bottom > m_curRect.bottom) m_curRect.bottom = rect.bottom;

}

void CMultiRectTracker::ExpandRect(CRect & srcRect, CRect & expRect)
{
	if (expRect.left   < srcRect.left)   srcRect.left   = expRect.left;
	if (expRect.right  > srcRect.right)  srcRect.right  = expRect.right;
	if (expRect.top    < srcRect.top)    srcRect.top    = expRect.top;
	if (expRect.bottom > srcRect.bottom) srcRect.bottom = expRect.bottom;
}


bool CMultiRectTracker::IsSelObjectsListEmpty()
{
	return m_selObjects.IsEmpty();
}

int CMultiRectTracker::Nearest(int nPos, int nGrid)
{
	int nGridPos = (nPos / nGrid) * nGrid;

	if (nGridPos > nPos)
		nGridPos -= nGrid;
	else
	if (nGridPos < nPos)
		nGridPos += nGrid;

	return nGridPos;
}

void CMultiRectTracker::UpdateSelObjectsPos(int nSizeXL, int nSizeYT, int nSizeXR, int nSizeYB)
{

	CTList<CTObject *>::iterator it = m_selObjects.begin();

	while (it != NULL)
	{
		CRect & rect = it->m_curRect;

		if (nSizeYT != nSizeYB && it->m_nType == CTRL_COMBOBOX)
			nSizeYB = nSizeYT = 0;

		if ((nSizeYT != nSizeYB || nSizeXL != nSizeXR) &&
			(it->m_nType == CTRL_POPUPMENU ||
			 it->m_nType == CTRL_MENU))
		{
			nSizeYB = nSizeYT = 0;
			nSizeXL = nSizeXR = 0;
		}

		rect.left   += nSizeXL;
		rect.right  += nSizeXR;
		rect.top    += nSizeYT;
		rect.bottom += nSizeYB;

		++it;
	}

}

void CMultiRectTracker::NormalizeSelObjectsPos()
{

	CTList<CTObject *>::iterator it = m_selObjects.begin();

	while (it != NULL)
	{
		it->m_curRect.NormalizeRect();
		++it;
	}

}
void CMultiRectTracker::StoreObjectsPrevPos()
{
	CTList<CTObject *>::iterator it = m_selObjects.begin();
	CRect rect;

	while (it != NULL)
	{
		it->m_oldRect = it->m_curRect;
		++it;
	}
}

void CMultiRectTracker::UpdateSelObjectsBkg(CWnd * pWnd, bool bCurOld)
{
	CTList<CTObject *>::iterator it = m_selObjects.begin();

	CRect rect;

	int nTWidth  = m_nTrackerWidth;
	int nTHeight = m_nTrackerHeight;

	CFormCreatorView * pView = (CFormCreatorView *) pWnd->GetParentFrame()->GetActiveView();

	while (it != NULL)
	{

		if (bCurOld)
			rect = GetObjectRealRect(*it, it->m_curRect);
		else
			rect = GetObjectRealRect(*it, it->m_oldRect);


		pWnd->InvalidateRect(CRect(rect.left - nTWidth, rect.top - nTHeight, rect.right + nTWidth, rect.top), true);
		pWnd->InvalidateRect(CRect(rect.left - nTWidth, rect.bottom, rect.right + nTWidth, rect.bottom + nTHeight), true);
		pWnd->InvalidateRect(CRect(rect.left - nTWidth, rect.top, rect.left, rect.bottom), true);
		pWnd->InvalidateRect(CRect(rect.right, rect.top, rect.right + nTWidth, rect.bottom), true);

		pWnd->UpdateWindow();
//		pWnd->InvalidateRect(rect, FALSE);
//		pWnd->UpdateWindow();


		++it;
	}


}

int CMultiRectTracker::HitTest(CPoint point, CRect rect)
{
	int iMouseIn = 0;

	m_bHitMove = false;

	memset(piMousePos, 0, piMaxMousePos * sizeof(int));

	if (point.x >= rect.left   - m_nTrackerWidth)  piMousePos[iLeftMax]   = 1;
	if (point.x <= rect.right  + m_nTrackerWidth)  piMousePos[iRightMax]  = 1;
	if (point.y >= rect.top    - m_nTrackerHeight) piMousePos[iTopMax]    = 1;
	if (point.y <= rect.bottom + m_nTrackerHeight) piMousePos[iBottomMax] = 1;

	if (point.x <= rect.left)    piMousePos[iLeftMin]   = 1;
	if (point.x >= rect.right)   piMousePos[iRightMin]  = 1;
	if (point.y <= rect.top)     piMousePos[iTopMin]    = 1;
	if (point.y >= rect.bottom)  piMousePos[iBottomMin] = 1;

	if (point.x >= (rect.left + rect.right) / 2 - m_nTrackerWidth / 2) piMousePos[iWidthMidMin] = 1;
	if (point.x <= (rect.left + rect.right) / 2 + m_nTrackerWidth / 2) piMousePos[iWidthMidMax] = 1;

	if (point.y >= (rect.top + rect.bottom) / 2 - m_nTrackerHeight / 2) piMousePos[iHeightMidMin] = 1;
	if (point.y <= (rect.top + rect.bottom) / 2 + m_nTrackerHeight / 2) piMousePos[iHeightMidMax] = 1;

	for (int i = 0; i < ciMaxMousePos; i++)
	{
		iMouseIn = piMousePos[ciMousePos[i].iBit[0]] +
		           piMousePos[ciMousePos[i].iBit[1]] +
		           piMousePos[ciMousePos[i].iBit[2]] +
		           piMousePos[ciMousePos[i].iBit[3]];

		
		if (iMouseIn == 4)
		{
			m_nHitTest = ciMousePos[i].iRet;

			if (m_nHitTest == hitLeft  ||
				m_nHitTest == hitRight ||
				m_nHitTest == hitTop   ||
				m_nHitTest == hitBottom)

				m_bHitMove = true;

			if (ciMousePos[i].nMsk == m_nMask)
				m_nHitTest = hitNothing;

			return m_nHitTest;
		}
	}

	m_nHitTest = hitNothing;
	return hitNothing;
}

int CMultiRectTracker::MultiHitTest(CPoint point)
{
	CTList<CTObject *>::iterator it = m_selObjects.begin();

	while (it != NULL)
	{
		int nHitTest = HitTest(point, GetObjectRealRect(*it, it->m_curRect));

		if (nHitTest != hitNothing)
			return nHitTest;

		++it;

	}
	return hitNothing;
}

BOOL CMultiRectTracker::SetCursor(CWnd *pWnd, UINT nHitTest)
{
	if (nHitTest != hitNothing)
		::SetCursor(ciMousePos[nHitTest].hCur);
	else
		::SetCursor(m_hcurArrow);
		
    return TRUE;
}

CRect CMultiRectTracker::CalcDragRect(CRect & rect)
{
	CRect dragRect;

	dragRect.left   = rect.left   - m_nTrackerWidth;
	dragRect.top    = rect.top    - m_nTrackerHeight;
	dragRect.right  = rect.right  + m_nTrackerWidth;
	dragRect.bottom = rect.bottom + m_nTrackerHeight;

	return dragRect;
}
void CMultiRectTracker::Swap(int & iSrc, int & iDst)
{
	int iTemp = iSrc;
	iSrc = iDst;
	iDst = iTemp;
}

bool CMultiRectTracker::PosChange()
{
	if (m_curRect != m_oldRect)
		return true;
	else
		return false;
}

void CMultiRectTracker::SetClipRect(CRect & clpRect)
{
	m_clpRect = clpRect;
}

void CMultiRectTracker::SetMask(int nMask)
{
	m_nMask = nMask;
	
}
bool CMultiRectTracker::Track(CWnd *pWnd, CPoint point, int nHitTest)
{
	
	CFormCreatorView * pView = (CFormCreatorView *) pWnd->GetParentFrame()->GetActiveView();

	AfxLockTempMaps(); 
	
	pWnd->SetCapture();
	ASSERT(pWnd == CWnd::GetCapture());
	pWnd->UpdateWindow();

	int nOldX = point.x;
	int nOldY = point.y;
	
	m_nMousePosX = nOldX;
	m_nMousePosY = nOldY;

	int nNewX, nNewY;
	int nSizeX, nSizeY;

	CRect curRect;
	CRect tmpRect;

	CDC * pDC = pWnd->GetDC();

	ASSERT_VALID(pDC);

	bool bLBDn      = false;
	bool bFirstDraw = true;
	bool bFirstIn   = true;
	bool bMove      = false;

	int nSizeXL, nSizeXR;
	int nSizeYT, nSizeYB;

	CString str;
	
	if (m_nMask & ciMousePos[nHitTest].nMsk)
	{
		pWnd->ReleaseDC(pDC);
		ReleaseCapture();
		AfxUnlockTempMaps(FALSE);
		return false;
	}

	CTList <CTObject *>::iterator it;

	/* ustawienie flag kierunku ruchu myszki. Kady kierunek zapisamy jest
	   w tablicy ciMousePos. Np kliknicie lewego grnego roku prostokta
	   to ruchy w lewo i w gr */

	bool bLeftChange   = ciMousePos[nHitTest].iChg[0];
	bool bTopChange    = ciMousePos[nHitTest].iChg[1];
	bool bRightChange  = ciMousePos[nHitTest].iChg[2];
	bool bBottomChange = ciMousePos[nHitTest].iChg[3];


	CRect minRect(m_clpRect.left   + _abs(point.x - m_curRect.left),
		          m_clpRect.top    + _abs(point.y - m_curRect.top),
			      m_clpRect.right  - _abs(point.x - m_curRect.right),
				  m_clpRect.bottom - _abs(point.y - m_curRect.bottom));


	m_oldRect = m_curRect;

	MSG msg;

	int cPasses = 0;

	for (;;)
	{
		if (CWnd::GetCapture() != pWnd)
			break;
		
		VERIFY(::GetMessage(&msg, NULL, 0, 0));

		switch (msg.message)
		{
			case WM_LBUTTONUP:

					goto ExitProc;

					break;

					
			case WM_MOUSEMOVE:

					
					if (msg.wParam == 0) goto ExitProc;


					nNewX = (int) (short) LOWORD(msg.lParam);
					nNewY = (int) (short) HIWORD(msg.lParam);

					m_nMousePosX = nNewX;
					m_nMousePosY = nNewY;

					nSizeX = nNewX - nOldX;
					nSizeY = nNewY - nOldY;

	
					m_tmpRect = m_curRect;
	
					/* Zabezpieczenie przed odjezdzaniem kursora od selekcji. */

					if (nNewX > m_clpRect.right)  nNewX = m_clpRect.right;
					if (nNewY > m_clpRect.bottom) nNewY = m_clpRect.bottom;
					if (nNewX < m_clpRect.left)   nNewX = m_clpRect.left;
					if (nNewY < m_clpRect.top)    nNewY = m_clpRect.top;
	


					/* Zabezpieczenie przed moliwoci przeswania ramki, gdy
					   kursor myszki nie znajduje si w punkcie kliknicia na obiekcie. */

					if (nHitTest == hitMiddle && !m_bDlgSelected)
					{		

						if (nSizeX > 0 && nNewX < minRect.left)   nSizeX = 0;
						if (nSizeX < 0 && nNewX > minRect.right)  nSizeX = 0;
						if (nSizeY > 0 && nNewY < minRect.top)    nSizeY = 0;
						if (nSizeY < 0 && nNewY > minRect.bottom) nSizeY = 0;
					}
					
					/* "snap to edges" - jeli skok myszki by zbyt duy nie mona
					   uaktualni ramki o t warto (poniewa efekt bdzi taki,
					   e ruszamy szybko myszk ale obiekt nie przeswa si, powstanie
					   efekt brzydkich zaci). Dlatego obliczam maxymalne wartoci
					   o ktre mona przesun ramk. */

				
					if (bLeftChange)
					{	
						if (m_curRect.left   + nSizeX > m_clpRect.right)  nSizeX = m_clpRect.right  - m_curRect.left; else
						if (m_curRect.left   + nSizeX < m_clpRect.left)   nSizeX = m_clpRect.left   - m_curRect.left;
					
					} 
					if (bRightChange)
					{		
						if (m_curRect.right  + nSizeX > m_clpRect.right)  nSizeX = m_clpRect.right  - m_curRect.right; else
						if (m_curRect.right  + nSizeX < m_clpRect.left)   nSizeX = m_clpRect.left   - m_curRect.right;


					}
					if (bTopChange)
					{			
						if (m_curRect.top    + nSizeY > m_clpRect.bottom) nSizeY = m_clpRect.bottom - m_curRect.top; else
						if (m_curRect.top    + nSizeY < m_clpRect.top)    nSizeY = m_clpRect.top    - m_curRect.top;

					} 
					if (bBottomChange)
					{		
						if (m_curRect.bottom + nSizeY > m_clpRect.bottom) nSizeY = m_clpRect.bottom - m_curRect.bottom; else
						if (m_curRect.bottom + nSizeY < m_clpRect.top)    nSizeY = m_clpRect.top    - m_curRect.bottom;

					}

		
					nSizeXL = nSizeXR = nSizeX;
					nSizeYT = nSizeYB = nSizeY;

	
					if (bLeftChange)   m_curRect.left   += nSizeXL;	else nSizeXL = 0;
					if (bTopChange)    m_curRect.top    += nSizeYT;	else nSizeYT = 0;
					if (bRightChange)  m_curRect.right  += nSizeXR; else nSizeXR = 0;
					if (bBottomChange) m_curRect.bottom += nSizeYB;	else nSizeYB = 0;

					
					if (m_curRect != m_tmpRect)
					{		
						if (bFirstDraw)
						{
							
							if (!m_bDlgSelected)
							{

								UpdateSelObjectsBkg(pWnd, true);
								pView->DrawNumbers();
								
								StoreObjectsPrevPos();
							}
							else
								pView->EraseDlgDragRect();

							/* obiekt zosta zaznaczony ale kursor si nie zmieni 
							   (nie zdya si wywoa funkcja HitTest) */

							if (m_nHitTest == hitNothing || m_nHitTest == hitMiddle)
								SetCursor(pWnd, hitMiddle);
						}

						bMove = true;

						CRect curRect = m_curRect;
						curRect.NormalizeRect();
						QuantizeRect(curRect, true);

						if (m_bDlgSelected)
							DrawSelectionRect(pDC, bFirstDraw);
						else
						{
							UpdateSelObjectsPos(nSizeXL, nSizeYT, nSizeXR, nSizeYB);
							DrawMultiSelectionRect(pDC, &m_selObjects, bFirstDraw);
							curRect = GetObjectRect((*m_selObjects.begin())->m_pTParent, curRect);
						}

						pView->UpdateObjectPosInfo(curRect);
	
						bFirstDraw = false;

					}


					nOldX = nNewX;
					nOldY = nNewY;
					
					break;

			default:
					DispatchMessage(&msg);
					break;
		}


	}

ExitProc:

	/* Usuniecie ramki. Tylko w przypadku gdy nastpi ruch myszk (flaga bMove)*/

	if (bMove)
	{
	
		if (m_bDlgSelected)
			DrawSelectionRect(pDC, true);
		else
			DrawMultiSelectionRect(pDC, &m_selObjects, true);
	}
	
	pWnd->ReleaseDC(pDC);
	ReleaseCapture();
	AfxUnlockTempMaps(FALSE);


	bool bPosChange;

	/* okrelenie czy pozycja ramki si zmienia */
	
	if (m_curRect != m_oldRect)
	{
		bPosChange = true;
		/* uaktualnienie pozycji ramek z uwzgldnieniem siatki */

		QuantizeObjectsPos();
		QuantizeRect(m_curRect, true);

		/* znormalizowanie pozycji ramek (grny lewy rg < dolny prawy rg */

		m_curRect.NormalizeRect();
		NormalizeSelObjectsPos();
	}
	else
		bPosChange = false;


	return bPosChange;
}


BOOL CMultiRectTracker::TrackRubberBand(CWnd *pWnd, CPoint point)
{
	if (::GetCapture() != NULL)
		return FALSE;

	CFormCreatorView * pView = (CFormCreatorView *) pWnd->GetParentFrame()->GetActiveView();

	AfxLockTempMaps(); 

	pWnd->SetCapture();
	ASSERT(pWnd == CWnd::GetCapture());
	pWnd->UpdateWindow();

	::SetCursor(m_hcurArrow);

	point.x = Nearest(point.x, m_nGridX);
	point.y = Nearest(point.y, m_nGridY);

	int nOldX = point.x;
	int nOldY = point.y;

	int nNewX, nNewY;
	int nSizeX, nSizeY;

	CDC * pDC = pWnd->GetDC();

	bool bMove = false;
	bool bFirstIn = true;

	CString str;

	m_curRect.SetRect(point.x, point.y, point.x, point.y);

	pWnd->GetClientRect(&m_clpRect);

	m_nHitTest = hitBottomRight;

	MSG msg;

	for (;;)
	{
		if (CWnd::GetCapture() != pWnd)
			break;

		VERIFY(::GetMessage(&msg, NULL, 0, 0));

		switch (msg.message)
		{
			case WM_LBUTTONUP:
					
					DrawSelectionRect(pDC, true);

					goto ExitProc;

			case WM_MOUSEMOVE:
            
					if (msg.wParam == 0) goto ExitProc;

					bMove = true;

					m_tmpRect = m_curRect;

					nNewX = (int) (short) LOWORD(msg.lParam);
					nNewY = (int) (short) HIWORD(msg.lParam);

					/* zabezpieczenie przed odjezdzaniem kursora od selekcji */

					if (nNewX > m_clpRect.right)  nNewX = m_clpRect.right;
					if (nNewY > m_clpRect.bottom) nNewY = m_clpRect.bottom;
					if (nNewX < m_clpRect.left)   nNewX = m_clpRect.left;
					if (nNewY < m_clpRect.top)    nNewY = m_clpRect.top;


					nSizeX = nNewX - nOldX;
					nSizeY = nNewY - nOldY;

					if (m_curRect.right  + nSizeX > m_clpRect.right)  nSizeX = m_clpRect.right  - m_curRect.right; else
					if (m_curRect.right  + nSizeX < m_clpRect.left)   nSizeX = m_clpRect.left   - m_curRect.left;
					if (m_curRect.bottom + nSizeY > m_clpRect.bottom) nSizeY = m_clpRect.bottom - m_curRect.bottom; else
					if (m_curRect.bottom + nSizeY < m_clpRect.top)    nSizeY = m_clpRect.top    - m_curRect.top;

					m_curRect.bottom += nSizeY;
					m_curRect.right  += nSizeX;

					DrawSelectionRect(pDC, false);

					pView->UpdateObjectPosInfo(m_qCurRect);


					nOldX = nNewX;
					nOldY = nNewY;


				break;

			case WM_KEYDOWN:
					break;
			default:
					DispatchMessage(&msg);
		}
	}

ExitProc:
	
	pWnd->ReleaseDC(pDC);
	ReleaseCapture();
	AfxUnlockTempMaps(FALSE);

	if (!bMove)
		m_curRect.SetRect(point.x, point.y, point.x, point.y);
	else
	{
		m_curRect = m_qCurRect;
		m_curRect.NormalizeRect();
	}

	return TRUE;
}



