#pragma once

#include "CTList.h"
#include "TObject.h"

const int hitNothing      = 13;
const int hitTopLeft      = 0;
const int hitBottomLeft   = 1;
const int hitTopRight     = 2;
const int hitBottomRight  = 3;
const int hitMiddleLeft   = 4; 
const int hitMiddleRight  = 5; 
const int hitMiddleTop    = 6; 
const int hitMiddleBottom = 7; 
const int hitTop          = 8;
const int hitBottom       = 9;
const int hitLeft         = 10;
const int hitRight        = 11;
const int hitMiddle       = 12;  

const int mskTopLeft      = 1;
const int mskBottomLeft   = 2;
const int mskTopRight     = 4;
const int mskBottomRight  = 8;
const int mskMiddleLeft   = 16;
const int mskMiddleRight  = 32; 
const int mskMiddleTop    = 64; 
const int mskMiddleBottom = 128; 
const int mskTop          = 256;
const int mskBottom       = 512;
const int mskLeft         = 1024;
const int mskRight        = 2048;
const int mskMiddle       = 4096;  
const int mskNothing      = 0;
const int mskAll          = 0xFFFFFFFF;
const int mskWidth        = mskTopLeft    | mskTopRight    | mskMiddleTop |
                            mskBottomLeft | mskBottomRight | mskMiddleBottom;
const int mskHeight       = mskTopLeft    | mskTopRight    | mskMiddleLeft |
                            mskBottomLeft | mskBottomRight | mskMiddleRight;
const int mskSize         = mskTopLeft    | mskTopRight    | mskMiddleLeft  |
                            mskBottomLeft | mskBottomRight | mskMiddleRight |
							mskMiddleTop  | mskMiddleBottom;
const int mskMove	      = mskLeft | mskRight | mskTop | mskBottom;




class CMultiRectTracker
{

public:
	CMultiRectTracker();
	~CMultiRectTracker();

	void DrawMultiSelectionRect(CDC* pDC, CTList<CTObject *> *objects, bool bCurTmp);

	void DrawSelectionRect(CDC* pDC, bool bDrawOnly);

	void DrawDottedRect(CDC * pDC, int x0, int y0, int x1, int y1);

	void DrawMultiTrackerRect(CDC * pDC);
	void DrawTrackerRect(CDC * pDC, CRect rect, bool bIsLast);
	
	CRect GetObjectTotalRect(CTList<CTObject *> * objects, CRect & tltRect);
	void  InvalidateNumRects(CTList<CTObject *> * objects, CRect * tltRect = NULL);

	int  HitTest(CPoint point, CRect rect);
	int  MultiHitTest(CPoint point);

	bool AddObject(CTObject * pObject);
	void RemoveAllObjects();
	bool IsSelObjectsListEmpty();

	void SetMask(int nMask);
//	void ClearTrackerRect(CDC * pDC);

	BOOL SetCursor(CWnd* pWnd, UINT nHitTest);	

	void SetScrollPos(int nScrollPosX, int nScrollPosY);
	void SetClipRect(CRect & clpRect);
	
	bool Track(CWnd* pWnd, CPoint point, int nHitTest);
	BOOL TrackRubberBand(CWnd *pWnd, CPoint point);

	bool PosChange();
	void MoveSelObjects(CWnd * pWnd);
	bool IntersectRect(CRect & rect0, CRect & rect1);

	void UpdateSelObjectsPos(int nSizeXL, int nSizeYT, int nSizeXR, int nSizeYB);
	void UpdateSelObjectsBkg(CWnd * pWnd, bool bCurOld);
	void QuantizeObjectsPos();
	void StoreObjectsPrevPos();

	void QuantizeRect(CRect & rect, bool bCurOld);
	void NormalizeSelObjectsPos();

	void RecalculateSelctionRect();
	void ExpandRect(CRect & rect);
	void ExpandRect(CRect & srcRect, CRect & expRect);
	void SetRect(CRect & rect);
	CRect CalcDragRect(CRect & rect);
	CBrush * LoadDIBBrush(int IDB_BRUSH);

	void Swap(int & iSrc, int & iDst);
	int  Nearest(int nPos, int nGrid);

	void SetGrid(int nGridX, int nGridY);

    CTList <CTObject *> m_selObjects; //selected objects
	CTList <CTObject *> * m_objects;
	typedef CTList<CTObject *>::iterator objSelIterator;
	CRect GetObjectRealRect(CTObject * pTObject, CRect & rect);
	CRect GetObjectRect(CTObject * pTObject, CRect & rect);


protected:

public:
	int       m_nTrackerWidth;
	int		  m_nTrackerHeight;

	int       m_nGridX;
	int       m_nGridY;

	int       m_nMousePosX;
	int       m_nMousePosY;

	int       m_nObjSelected;

	int       m_nMask;

	CRect     m_curRect;
	CRect     m_oldRect;
	CRect     m_clpRect;
	CRect     m_tmpRect;

	CRect     m_qCurRect;

	HCURSOR   m_hcurArrow;
	HCURSOR   m_hcurMove;
	HCURSOR   m_hcurSize1;
	HCURSOR   m_hcurSize2;
	HCURSOR   m_hcurSize3;
	HCURSOR   m_hcurSize4;

	bool m_bMultiSelection;
	bool m_bAllowInvert;
	bool m_bDlgSelected;
	bool m_bDragNDrop;

	/* Flaga ustawiana gdy kliknito w rodku prostokta,
	   ale nie na punktach kontrolnych */

	bool m_bHitMove;
	int  m_nHitTest;

	CWnd * m_pParent;

};

